#ifndef _KTEXTBOX_H
#define _KTEXTBOX_H

/******************************************************************************
**
**  $Id: ktextbox.h,v 1.4 1994/11/21 22:06:49 vertex Exp $
**
******************************************************************************/


#include "ktextbox_error.h"

#define justLeft 		0 /*  */
#define justRight 		1 /*  */
#define justCenter 		2 /* E */
#define justJustify 		3 /* ϓ */

#define SJIS_KANJI_HIGH(c)      (((0x81 <= (c)) && ((c) <= 0x9f)) || ((0xe0 <= (c)) && ((c) <= 0xfc)))
#define SJIS_KANJI_LOW(c)       (((0x40 <= (c)) && ((c) <= 0x7e)) || ((0x80 <= (c)) && ((c) <= 0xfc)))
#define SJIS_KANJI_LEVEL2(c)    ((0x989f <= (c)) && ((c) <= 0xeafc))
#define NUM_KANJI_LEVEL1    	3760

#define fg_Opaque		0x01 /* Fɕsݒ */
#define bg_Opaque		0x02 /* wiFɕsݒ */

typedef struct Rectf16 {
	frac16 rectf16_XLeft;
	frac16 rectf16_YTop;
	frac16 rectf16_XRight;
	frac16 rectf16_YBottom;
} Rectf16;

typedef struct 
{
    short	align;		/* 񂹁AAE*/
						/* Text alignment (left, right, center). */
    int32	charPitch;	/* ԁihbgPʁj*/
						/* Inter-character spacing. */
    int32	linePitch;	/* sԁihbgPʁj*/
						/* Spacing bewteen adjactent rows (leading). */
} TextAlign ;

typedef struct
{
    Rectf16		celRect;        /* Z̑傫w */
								/* Cel bounds. */
    Rectf16     wrapRect;       /* eLXg`̈w */
								/* Bounds to wrap text into. */
    uint16      align;          /* eLXg̑̎w */
								/* Text alignment (left, right, center, justify). */
    int32       charPitch;      /* Ԃw */
								/* Inter-character spacing. */
    int32       linePitch;      /* sԂw */
								/* Spacing bewteen adjactent rows (leading). */
    Color       fgColor;        /* Fw */
								/* Foreground color (text color). */
    Color       bgColor;        /* wiFw */
								/* Background color (color beneath text). */
    uint8		opaqueFlag;		/* sw */
								/* Is the text background opaque? */
    uint8       otherFlags;     /* \\ */
								/* Reserved.  Set to NULL. */
    int32       reserved;     	/* \\ */
								/* Reserved.  Set to NULL. */
} KTextBox;

typedef struct {
    uint16     	nChars;			/* ܂܂镶R[h̐*/
								/* Number of characters. */
    uint16     	minCode;		/* ŏ̕R[h*/
								/* Minimum character code. */
    uint16     	maxCode;		/* ő̕R[h*/
								/* Maximum character code. */
    uint8		charHeight; 	/* ̍(pixels) */
								/* Character height (pixels). */
    uint8		charWidth; 		/* ̕(pixels) */
								/* Character width (pixels). */
    uint32		oneCharSize;	/* P̃sNZf[^̃oCg(pad byte܂) */
								/* Size of one character bitmap in bytes (long-word aligned). */
    uint16		*codeTbl;		/* R[he[uւ̃|C^ */
								/* Optional character code table. */
    uint8		*widthTbl;		/* σsb`̕e[uւ̃|C^ */
								/* Optional caracter width table. */
    char 		*charData;		/* sNZf[^̐擪|C^*/
								/* Character bitmap data. */
} KFontRec;

typedef struct {
    uint8     	font_Gpp;			/* Gray scale bits per pixel*/
    uint8     	font_Bpp;			/* Bits per pixel */
    KFontRec	font_Hinfo;			/* ppFontRec\\ */
									/* Hankaku (half-width) character information. */
    KFontRec	font_Zinfo;			/* SppFontRec\\ */
									/* Zenkaku (full-width) character information. */
	Stream*		font_fs;			/* Font file stream */
	Boolean		font_FullRead;		/* If true, read whole font image into memory. */
	uint32		font_HFSeekOffset;	/* File offset to Hankaku bitmap data. */
	uint32		font_ZFSeekOffset;	/* File offset to Zenkaku bitmap data. */
	Boolean		fsRead;				/* If true, use file system calls; otherwise, use low-level IO calls. */
	uint32		maxCharSize;		/* Maximum char size */
	IOInfo		romioInfo;			/* Internal use only. */
	Item		romIOReqItem;		/* Internal use only. */
	IOReq		*romior;			/* Internal use only. */
	uint32		logRomBlockSize;	/* Internal use only. */
} KFont3DO;

extern int32 KLoadFont(char *name, KFont3DO *theFont, Boolean fullRead );

extern int32 KFreeFont(KFont3DO *theFont);

extern CCB * KTextBox_A(KFont3DO *theFont, uint8 *theText, int32 textLen, Rectf16 *wrapBox, TextAlign *theAlign, Color fgColor, Color bgColor, CCB *userCCB);

extern int32 KTextBox_B(KFont3DO *theFont, KTextBox *theTextBox, uint8 *theText, int32 textLen, CCB **userCCB);

extern int32 KTextLength( uint8 *theText );

extern int32 KFreeCel( CCB *theCCB );

extern int32 KGetCharWidth(KFont3DO *theFont, uint16 charVal);

extern int32 KGetCharHeight(KFont3DO *theFont, uint16 charVal);

#endif /* _KTEXTBOX_H */
