/* $Id: random.c,v 1.18 1994/09/10 02:22:15 vertex Exp $ */

#include "types.h"
#include "kernel.h"
#include "clio.h"
#include "inthard.h"
#include "time.h"
#include "internalf.h"

/**
|||	AUTODOC PUBLIC spg/kernel/readhardwarerandomnumber
|||	ReadHardwareRandomNumber - Get a 32-bit random number.
|||
|||	  Synopsis
|||
|||	    uint32 ReadHardwareRandomNumber( void )
|||
|||	  Description
|||
|||	    The procedure returns a random number generated by the hardware.
|||	    This is useful for providing a seed to the software random-number
|||	    generator.
|||
|||	  Return Value
|||
|||	    The procedure returns an unsigned 32-bit random number.  The
|||	    number can be any integer from 0 to (2^32 - 1), inclusive.
|||
|||	  Implementation
|||
|||	    SWI implemented in kernel folio V20.
|||
|||	  Associated Files
|||
|||	    hardware.h                  ARM C "swi" declaration
|||
|||	  See Also
|||
|||	    srand(), rand(), urand()
|||
**/

uint32
ReadHardwareRandomNumber(void)
{
	uint32 lrand, rrand;
	Clio *clio = (Clio *)CLIO;

/*
 * back-to-back reads of DSPPNoise
 * may not be completely random; so
 * we read it a couple extra times and
 * discard the middle values.
 */
	lrand = clio->clio_DSPPNoise;
	rrand = clio->clio_DSPPNoise;
	rrand = clio->clio_DSPPNoise;
	rrand = clio->clio_DSPPNoise;

	return (lrand << 16) | rrand;
}
